import { reactive, ref, computed } from 'vue'
import { PAGE_SIZE_OPTIONS } from '/@/config'

export default class TableHelper {
  // 当前页
  currentPage = ref(1)
  // 每页条数
  pageSize = ref(Number(PAGE_SIZE_OPTIONS[0]))
  // 搜索的关键词
  searchText = ref('')
  // 表格数据
  tableData = ref([])
  // 数据总数
  total = ref(0)
  // 数据是否正在加载
  loading = ref(false)
  // 请求参数，非必须字段，由外部传入
  otherParams = reactive({})
  // 请求参数
  requestParams = reactive({
    pageNo: this.currentPage,
    pageSize: this.pageSize
  })

  constructor (requestFn, initRequestParams) {
    this.requestFn = requestFn
    this.initRequestParams = initRequestParams
    if (this.initRequestParams) Object.assign(this.otherParams, this.initRequestParams)
    this.fetchTableData()
  }

  params = computed(() => {
    return {
      ...this.requestParams,
      ...this.otherParams
    }
  })

  /**
   * 根据声明的请求方法和参数获取表格分页数据
   * @param extraRequestParams 允许传入额外的数据，将通过解构的方式混入到params里
   */
  fetchTableData () {
    this.loading.value = true
    return new Promise((resolve, reject) => {
      this.requestFn(this.params.value).then((res) => {
        this.tableData.value = res.records
        this.total.value = res.total
        resolve(this)
      }).catch(e => {
        this.tableData.value = []
        this.total.value = 0
        reject(e)
      }).finally(() => {
        this.loading.value = false
      })
    })
  }

  /**
   * 翻页
   * @param page
   */
  async onPageChange (page) {
    this.currentPage.value = page
    await this.fetchTableData()
  }

  /**
   * 切换每页条数
   * @param pageSize
   */
  async onPageSizeChange (pageSize) {
    this.currentPage.value = 1
    this.pageSize.value = pageSize
    await this.fetchTableData()
  }

  /**
   * 搜索
   * @param extraRequestParams
   */
  async onSearch (extraRequestParams) {
    this.currentPage.value = 1
    Object.assign(this.otherParams, extraRequestParams)
    await this.fetchTableData()
  }

  /**
   * 刷新数据
   * 当前页刷新数据，保留所有条件
   */
  async refreshTableData () {
    await this.fetchTableData()
  }

  /**
   * 重置数据
   * 回到第一页
   * 清空搜索关键词
   */
  async resetTableData () {
    this.currentPage.value = 1
    this.searchText.value = ''
    await this.fetchTableData()
  }
}
